using Adaptation.Eaf.Management.ConfigurationData.CellAutomation;
using Adaptation.Ifx.Eaf.EquipmentConnector.File.Configuration;
using Adaptation.Shared;
using Adaptation.Shared.Duplicator;
using Adaptation.Shared.Methods;
using Adaptation.Shared.Metrology;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;

namespace Adaptation.FileHandlers.OpenInsight;

public class FileRead : Shared.FileRead, IFileRead
{

    private string _LastLines;
    private readonly string _IqsConnectionString;
    private readonly string _OpenInsightFilePattern;
    private readonly string _OpenInsightApiECDirectory;

    public FileRead(ISMTP smtp, Dictionary<string, string> fileParameter, string cellInstanceName, int? connectionCount, string cellInstanceConnectionName, FileConnectorConfiguration fileConnectorConfiguration, string equipmentTypeName, string parameterizedModelObjectDefinitionType, IList<ModelObjectParameterDefinition> modelObjectParameters, string equipmentDictionaryName, Dictionary<string, List<long>> dummyRuns, Dictionary<long, List<string>> staticRuns, bool useCyclicalForDescription, bool isEAFHosted) :
        base(new Description(), false, smtp, fileParameter, cellInstanceName, connectionCount, cellInstanceConnectionName, fileConnectorConfiguration, equipmentTypeName, parameterizedModelObjectDefinitionType, modelObjectParameters, equipmentDictionaryName, dummyRuns, staticRuns, useCyclicalForDescription, isEAFHosted: connectionCount is null)
    {
        _MinFileLength = 10;
        _Logistics = new(this);
        _NullData = string.Empty;
        if (_FileParameter is null)
            throw new Exception(cellInstanceConnectionName);
        if (_ModelObjectParameterDefinitions is null)
            throw new Exception(cellInstanceConnectionName);
        if (!_IsDuplicator)
            throw new Exception(cellInstanceConnectionName);
        _LastLines = string.Empty;
        _IqsConnectionString = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "IQS.ConnectionString");
        _OpenInsightApiECDirectory = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "API.EC.Directory");
        _OpenInsightFilePattern = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "OpenInsight.FilePattern");
    }

    void IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, Exception exception)
    {
        bool isErrorFile = exception is not null;
        if (!isErrorFile && !string.IsNullOrEmpty(_Logistics.ReportFullPath))
        {
            FileInfo fileInfo = new(_Logistics.ReportFullPath);
            if (fileInfo.Exists && fileInfo.LastWriteTime < fileInfo.CreationTime)
                File.SetLastWriteTime(_Logistics.ReportFullPath, fileInfo.CreationTime);
        }
        Move(extractResults);
    }

    void IFileRead.WaitForThread() => WaitForThread(thread: null, threadExceptions: null);

    string IFileRead.GetEventDescription()
    {
        string result = _Description.GetEventDescription();
        return result;
    }

    List<string> IFileRead.GetHeaderNames()
    {
        List<string> results = _Description.GetHeaderNames();
        return results;
    }

    string[] IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, string to, string from, string resolvedFileLocation, Exception exception)
    {
        string[] results = Move(extractResults, to, from, resolvedFileLocation, exception);
        return results;
    }

    JsonProperty[] IFileRead.GetDefault()
    {
        JsonProperty[] results = _Description.GetDefault(this, _Logistics);
        return results;
    }

    Dictionary<string, string> IFileRead.GetDisplayNamesJsonElement()
    {
        Dictionary<string, string> results = _Description.GetDisplayNamesJsonElement(this);
        return results;
    }

    List<IDescription> IFileRead.GetDescriptions(IFileRead fileRead, List<Test> tests, IProcessData processData)
    {
        List<IDescription> results = _Description.GetDescriptions(fileRead, _Logistics, tests, processData);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.GetExtractResult(string reportFullPath, string eventName)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        if (string.IsNullOrEmpty(eventName))
            throw new Exception();
        _ReportFullPath = reportFullPath;
        DateTime dateTime = DateTime.Now;
        results = GetExtractResult(reportFullPath, dateTime);
        if (results.Item3 is null)
            results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(results.Item1, Array.Empty<Test>(), JsonSerializer.Deserialize<JsonElement[]>("[]"), results.Item4);
        if (results.Item3.Length > 0 && _IsEAFHosted)
            WritePDSF(this, results.Item3);
        UpdateLastTicksDuration(DateTime.Now.Ticks - dateTime.Ticks);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.ReExtract()
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        List<string> headerNames = _Description.GetHeaderNames();
        Dictionary<string, string> keyValuePairs = _Description.GetDisplayNamesJsonElement(this);
        results = ReExtract(this, headerNames, keyValuePairs);
        return results;
    }

    internal static Tuple<string, string> GetLines(Logistics logistics, List<txt.Description> descriptions)
    {
        StringBuilder result = new();
        char del = '\t';
        txt.Description x = descriptions[0];
        _ = result.Append(x.DcnLpdMin).Append(del).    // 001 - 
            Append(x.DcnLpdMax).Append(del).       // 002 - 
            Append(x.DcnLpdMean).Append(del).      // 003 - DCN LPD
            Append(x.DcnAreaCountMin).Append(del). // 004 - 
            Append(x.DcnAreaCountMax).Append(del). // 005 - 
            Append(x.DcnAreaCountMean).Append(del).// 006 - DCN Area
            Append(x.DcnAreaMin).Append(del).      // 007 - 
            Append(x.DcnAreaMax).Append(del).      // 008 - 
            Append(x.Date).Append(del).            // 009 - 
            Append(x.DcnHazeAvgMean).Append(del).  // 010 - Haze Average
            Append(string.Empty).Append(del).              // 011 - 
            Append(string.Empty).Append(del).              // 012 - 
            Append(string.Empty).Append(del).              // 013 - 
            Append(string.Empty).Append(del).              // 014 - 
            Append(string.Empty).Append(del).              // 015 - 
            Append(string.Empty).Append(del).              // 016 - 
            Append(string.Empty).Append(del).              // 017 - 
            Append(string.Empty).Append(del).              // 018 - 
            Append(string.Empty).Append(del).              // 019 - 
            Append(string.Empty).Append(del).              // 020 - 
            Append(string.Empty).Append(del).              // 021 - 
            Append(string.Empty).Append(del).              // 022 - 
            Append(string.Empty).Append(del).              // 023 - 
            Append(string.Empty).Append(del).              // 024 - 
            Append(string.Empty).Append(del).              // 025 - 
            Append(string.Empty).Append(del).              // 026 - 
            Append(string.Empty).Append(del).              // 027 - 
            Append(x.RDS).Append(del).             // 028 - Lot
            Append(x.Reactor).Append(del).         // 029 - Process
            Append(x.Recipe).Append(del).          // 030 - Part
            Append(x.DcnScrMean).Append(del).      // 031 - Scratch Count
            Append(string.Empty).Append(del).              // 032 - 
            Append(string.Empty).Append(del).              // 033 - 
            Append(string.Empty).Append(del).              // 034 - 
            Append(x.DcnMicroScrMean).Append(del). // 035 - Scratch Length
            Append(string.Empty).Append(del).              // 036 - 
            Append(string.Empty).Append(del).              // 037 - 
            Append(string.Empty).Append(del).              // 038 - 
            Append(x.DcnAllMean).Append(del).      // 039 - Average Sum of Defects
            Append(x.DcnAllMax).Append(del).       // 040 - Max Sum of defects
            Append(x.DcnAllMin).Append(del).       // 041 - Min Sum of Defects
            Append(string.Empty).Append(del).              // 042 - 
            Append(logistics.MesEntity).Append(del).                 // 043 - 
            Append(x.DcnAreaMean).Append(del).     // 044 - DCN MM2
            AppendLine();
        return new Tuple<string, string>(result.ToString(), x.Date);
    }

    private void SaveOpenInsightFile(string reportFullPath, DateTime dateTime, string logistics, List<txt.Description> descriptions, Test[] tests)
    {
        bool isDummyRun = false;
        List<(Shared.Properties.IScopeInfo, string)> collection = new();
        string duplicateDirectory = Path.Combine(_FileConnectorConfiguration.SourceFileLocation, _CellInstanceName);
        if (!Directory.Exists(duplicateDirectory))
            _ = Directory.CreateDirectory(duplicateDirectory);
        string successDirectory = _FileConnectorConfiguration.AlternateTargetFolder;
        if (!Directory.Exists(Path.Combine(duplicateDirectory, "1")))
        {
            string parentParent = GetParentParent(_FileConnectorConfiguration.SourceFileLocation);
            if (parentParent.Contains(_CellInstanceName))
                parentParent = Path.GetDirectoryName(parentParent);
            duplicateDirectory = Path.Combine(parentParent, "Data");
            if (!Directory.Exists(duplicateDirectory))
                _ = Directory.CreateDirectory(duplicateDirectory);
        }
        string duplicateFile = Path.Combine(duplicateDirectory, Path.GetFileName(reportFullPath));
        if (descriptions.Count == 0 || tests.Length == 0)
            _LastLines = string.Empty;
        else
        {
            Tuple<string, string> lines = GetLines(_Logistics, descriptions);
            string check = lines.Item1.Replace(lines.Item2, "$Date$");
            bool save = string.IsNullOrEmpty(_LastLines) || check != _LastLines;
            if (save && !string.IsNullOrEmpty(check))
            {
                long? subGroupId;
                _LastLines = check;
                long breakAfter = dateTime.AddSeconds(_BreakAfterSeconds).Ticks;
                long preWait = _FileConnectorConfiguration?.FileHandleWaitTime is null ? dateTime.AddMilliseconds(1234).Ticks : dateTime.AddMilliseconds(_FileConnectorConfiguration.FileHandleWaitTime.Value).Ticks;
                if (string.IsNullOrEmpty(descriptions[0].Reactor) || string.IsNullOrEmpty(descriptions[0].PSN))
                    subGroupId = null;
                else
                    (subGroupId, int? _, string _) = FromIQS.GetCommandText(_IqsConnectionString, _Logistics, descriptions[0], breakAfter, preWait);
                if (subGroupId is null)
                    collection.Add(new(new ScopeInfo(tests[0], _OpenInsightFilePattern), lines.Item1));
                else
                    collection.Add(new(new ScopeInfo(tests[0], $"{subGroupId.Value} {_OpenInsightFilePattern}"), lines.Item1));
                string weekOfYear = _Calendar.GetWeekOfYear(_Logistics.DateTimeFromSequence, CalendarWeekRule.FirstDay, DayOfWeek.Sunday).ToString("00");
                FromIQS.Save(_OpenInsightApiECDirectory, _Logistics, reportFullPath, logistics, descriptions.First(), lines.Item1, subGroupId, weekOfYear);
            }
            if (!Directory.Exists(duplicateDirectory))
                _ = Directory.CreateDirectory(duplicateDirectory);
            if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
                WaitForFileConsumption(dateTime, descriptions, isDummyRun, successDirectory, duplicateDirectory, collection, duplicateFile);
        }
    }

    private Tuple<string, Test[], JsonElement[], List<FileInfo>> GetExtractResult(string reportFullPath, DateTime dateTime)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        Tuple<string, string[], string[]> pdsf = ProcessDataStandardFormat.GetLogisticsColumnsAndBody(reportFullPath);
        _Logistics = new Logistics(reportFullPath, pdsf.Item1);
        SetFileParameterLotIDToLogisticsMID();
        JsonElement[] jsonElements = ProcessDataStandardFormat.GetArray(pdsf);
        List<txt.Description> descriptions = txt.ProcessData.GetDescriptions(jsonElements);
        Test[] tests = (from l in descriptions select (Test)l.Test).ToArray();
        if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
            SaveOpenInsightFile(reportFullPath, dateTime, pdsf.Item1, descriptions, tests);
        results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(pdsf.Item1, tests, jsonElements, new List<FileInfo>());
        return results;
    }

}