using Adaptation._Tests.Shared;
using Microsoft.Extensions.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Adaptation._Tests.Static;

[TestClass]
public class MET08DDUPSP1TBI : LoggingUnitTesting, IDisposable
{

#pragma warning disable CA2254
#pragma warning disable IDE0060

    internal static MET08DDUPSP1TBI LoggingUnitTesting { get; private set; }

    public MET08DDUPSP1TBI() : base(testContext: null, declaringType: null)
    {
        if (LoggingUnitTesting is null)
            throw new Exception();
    }

    public MET08DDUPSP1TBI(TestContext testContext) : base(testContext, new StackFrame().GetMethod().DeclaringType)
    {
    }

    [ClassInitialize]
    public static void ClassInitialize(TestContext testContext) => LoggingUnitTesting ??= new MET08DDUPSP1TBI(testContext);

    [ClassCleanup()]
    public static void ClassCleanup()
    {
        LoggingUnitTesting?.Logger?.LogInformation("Cleanup");
        LoggingUnitTesting?.Dispose();
    }

    private static void NonThrowTryCatch()
    {
        try
        { throw new Exception(); }
        catch (Exception) { }
    }

    [TestMethod]
    public void TestDateTime()
    {
        DateTime dateTime = DateTime.Now;
        Assert.IsTrue(dateTime.ToString("M/d/yyyy h:mm:ss tt") == dateTime.ToString());
    }

#if DEBUG
    [Ignore]
#endif
    [TestMethod]
    public void Production()
    {
        MethodBase methodBase = new StackFrame().GetMethod();
        StringBuilder results = new();
        (string cellInstanceName, string cellInstanceVersionName)[] collection = new (string, string)[]
        {
            new("MET08DDUPSP1TBI", "v2.57.0"),
        };
        string production = "http://messa08ec.infineon.com:9003/CellInstanceServiceV2";
        Shared.PasteSpecialXml.EAF.XML.API.CellInstance.CellInstanceVersion cellInstanceVersion;
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Getting configuration"));
        foreach ((string cellInstanceName, string cellInstanceVersionName) in collection)
        {
            cellInstanceVersion = AdaptationTesting.GetCellInstanceVersion($"{production}/{cellInstanceName}/{cellInstanceVersionName}/configuration");
            _ = results.AppendLine($"{cellInstanceName}\t{cellInstanceVersionName}\t{cellInstanceVersion.EdaConnection.PortNumber}");
        }
        string sourceDirectory = "D:/Tmp/cellInstanceVersion.EdaConnection.PortNumber";
        if (!Directory.Exists(sourceDirectory))
            _ = Directory.CreateDirectory(sourceDirectory);
        File.WriteAllText(Path.Combine(sourceDirectory, $"{methodBase.Module.Name}-{methodBase.ReflectedType.Name}-{methodBase.Name}.tsv"), results.ToString());
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
    }

    [TestMethod]
    public void CellInstanceVersionEdaConnectionPortNumber()
    {
        MethodBase methodBase = new StackFrame().GetMethod();
        string sourceDirectory = "D:/Tmp/cellInstanceVersion.EdaConnection.PortNumber";
        if (Directory.Exists(sourceDirectory))
        {
            string destinationDirectory = Path.Combine(sourceDirectory, "All");
            if (!Directory.Exists(destinationDirectory))
                _ = Directory.CreateDirectory(destinationDirectory);
            List<string> lines = new();
            string[] files = Directory.GetFiles(sourceDirectory, "*.tsv", SearchOption.TopDirectoryOnly);
            foreach (string file in files)
                lines.AddRange(File.ReadAllLines(file));
            File.WriteAllLines(Path.Combine(destinationDirectory, $"{DateTime.Now.Ticks}.tsv"), lines.Distinct().OrderBy(l => l));
        }
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
        NonThrowTryCatch();
    }

}